/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <kd@mnenhy.de>.
 *
 * Portions created by the Initial Developer are Copyright © 2002-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// +-
// |  the contents of this file will be loaded into the Mnenhy.headers.* context!
// |  ie. foo() is available as goMnenhy.headers.foo()
// +-


// |
// |  constants
// |
const gksViewPopup   = "mnenhyMenuPopupViewHeadersMenu";
const gksViewBaseID  = "urn:mnenhy:header:custom";
const gksViewCheck   = "checked";
const gksViewState   = "state";
const gksViewID      = "anonid";


// |
// |  functions
// |

// Since we need to copy the whole menupopup to every location that uses it,
// we might get multiple IDs. To avoid this (we can't for <template>, though),
// we're using a custom attribute named 'anonid' and the getElementsByAttribute function.
function InitMnenhyViewPopup(aoWin, aoElement)
{
  // lazily init the <template>
  if (!("mnenhyInit" in aoElement))
  {
    aoElement.setAttribute("datasources", goMnenhy.GetMnenhyRDF(true).spec);
    // do some magic for old Mozillas
    if (goMnenhy.ksMozillaVersion < "1.7")
    {
      try
      {
        // add mnenhy.rdf once
        aoElement.builder.database.AddDataSource(goMnenhy.m_oRDF.moDataSource);
        aoElement.builder.rebuild();
      }
      catch(oEx)
      {
        goMnenhy.Log(oEx);
      }
    }
    aoElement.mnenhyInit = true;
  }
  // set window (old current window may be closed by now!)
  SetCurrentWindow(aoWin);
  // select current item
  const ksInline    = "mnenhyViewInlineHeaders";
  const ksBrief     = "mnenhyViewBriefHeaders";
  const ksNone      = "mnenhyViewNoHeaders";
  const ksAll       = "mnenhyViewAllHeaders";
  const ksNormal    = "mnenhyViewNormalHeaders";
  const ksCustomSep = "mnenhyCustomizeSeparator";
  const ksCustomize = "mnenhyCustomizeHeaders";

  let sID = null;
  const kiHdrMode = GetHeaderView();
  if (kiHdrMode >= gkiViewCustomBase)
  {
    sID = gksViewBaseID + (kiHdrMode - gkiViewCustomBase);
  }
  else
  {
    switch (kiHdrMode)
    {
      // new non custom modes
      case gkiViewBriefMnenhy:                          // "brief" does not exist as a distinctive
      case gkiViewBriefHeaders:  sID = ksBrief;  break;  // option anymore, it's always there
      case gkiViewNoHeaders:     sID = ksNone;   break;
      // old modes
      case gkiViewAllHeaders:    sID = ksAll;    break;
      case gkiViewNormalHeaders:
      case gkiViewNormalMnenhy:
      default:                   sID = ksNormal; break;
    }
  }
  // checkdot active
  const kroMenuitem = aoElement.getElementsByAttribute(gksViewID, sID);
  if (kroMenuitem && kroMenuitem.length)
    kroMenuitem[0].setAttribute(gksViewCheck, "true");

  // checkmark inline
  const kroInline = aoElement.getElementsByAttribute(gksViewID, ksInline);
  if (kroInline && kroInline.length)
  {
    // set value
    const koInline = kroInline[0];
    koInline.setAttribute(gksViewCheck, GetHeaderViewInline());
    // allow manipulation
    if (AllowInline(kiHdrMode))
      koInline.removeAttribute("disabled");
    else
      koInline.setAttribute("disabled", "true");
  }

  // checkmark brief
  const kroBrief = aoElement.getElementsByAttribute(gksViewID, ksBrief);
  if (kroBrief && kroBrief.length)
  {
    if (goMnenhy.installed.IsThunderbird())
    {
      // TB3 has no brief header view anymore, thus we hide the menu
      kroBrief[0].setAttribute("hidden", true);
    }
    else
    {
      // set value
      kroBrief[0].setAttribute(gksViewCheck, GetHeaderViewBrief());
    }
  }

  // move separator and Customize... to the end of the menupopup
  let oMenuPopup, oItem;
  const kroCustomizeSep = aoElement.getElementsByAttribute(gksViewID, ksCustomSep);
  if (kroCustomizeSep && kroCustomizeSep.length)
  {
    oMenuPopup = kroCustomizeSep[0].parentNode;
    oItem = oMenuPopup.removeChild(kroCustomizeSep[0]);
    oMenuPopup.appendChild(oItem);
  }
  const kroCustomize = aoElement.getElementsByAttribute(gksViewID, ksCustomize);
  if (kroCustomize && kroCustomize.length)
  {
    oMenuPopup = kroCustomize[0].parentNode;
    oItem = oMenuPopup.removeChild(kroCustomize[0]);
    oMenuPopup.appendChild(oItem);
  }
}


// brief flag storage handling
function GetHeaderViewBrief()
{
  return ("gCollapsedHeaderViewMode" in goWin) && goWin.gCollapsedHeaderViewMode;
}
function ToggleHeaderViewBrief()
{
  // we must not call the "real" toggle function if no headers are visible
  const kbExpanded  = !IsAttributeTrue("expandedHeaderView",  gksViewClosed);
  const kbCollapsed = !IsAttributeTrue("collapsedHeaderView", gksViewClosed);
  let bSetToggleState = (!kbExpanded && !kbCollapsed);
  let bNewState = false;
  if (bSetToggleState)
  {
    // don't update the header view, just change the variables
    const koToggleHeaderView = goWin.document.getElementById(gksViewElement);
    bNewState = !IsAttributeTrue(gksViewElement, gksViewState);
    goWin.gCollapsedHeaderViewMode = bNewState;
  }
  else
  {
    // update view
    if ("ToggleHeaderView" in goWin)
      goWin.ToggleHeaderView();
    // TB2 uses a deck now...
    bSetToggleState = !!goWin.document.getElementById("msgHeaderViewDeck");
  }

  // manually set toggle state if necessary
  if (bSetToggleState)
  {
    if (goWin.gCollapsedHeaderViewMode)
      koToggleHeaderView.setAttribute(gksViewState, true); // set brief
    else
      koToggleHeaderView.setAttribute(gksViewState, false);
  }
}


// the "inline" menuitem MUST ONLY be active for gkiViewAllHeaders
function AllowInline(aiMode)
{
  return (aiMode == gkiViewAllHeaders);
}


// set/unset the inline message display
function SetInline(aoElement)
{
  const kbInline = aoElement.getAttribute(gksViewCheck) == "true";
  // store value
  SetHeaderViewInline(kbInline);
  // use value
  ProcessInline(kbInline);
  if (goWin.gDBView)
    goWin.ReloadMessage();
  return true;
}


// set specific view mode wrt inline and brief state
function SetView(aiMode)
{
  // the custom modes don't set an integer aiMode
  if (typeof(aiMode) == "object")
    aiMode = Number(aiMode.id.replace(gksViewBaseID, "")) + gkiViewCustomBase;
  // set view mode in prefs and mnenhy.rdf
  SetHeaderView(aiMode);
  // check for inline
  const kbInline = AllowInline(aiMode) && GetHeaderViewInline();
  ProcessInline(kbInline);
  // force rebuild
  goWin.giMnenhyHeaderView = gkiViewDirty;
  if (goWin.gDBView)
    goWin.ReloadMessage();
  return true;
}


// check if the collapsed attribute is set to true
function IsAttributeTrue(asElementID, asAttributeName)
{
  const koElement = goWin.document.getElementById(asElementID);
  if (koElement)
    return (koElement.getAttribute(asAttributeName) == "true");
  return false;
}
